package com.io.yutian.elementoriginlib.redis;

import com.google.common.net.InetAddresses;
import com.io.yutian.elementoriginlib.ElementOriginLib;
import org.json.JSONObject;
import redis.clients.jedis.Jedis;

import java.net.InetAddress;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

public class RedisBungeeUtil {

    public static UUID getUUIDFromRedis(String username) {
        try (Jedis jedis = ElementOriginLib.inst().getRedisIO().getJedisPool().getResource()) {
            if (jedis.exists("uuid-cache")) {
                String result = jedis.hget("uuid-cache", username.toLowerCase());
                JSONObject jsonObject = new JSONObject(result);
                String uuidString = jsonObject.getString("uuid");
                return UUID.fromString(uuidString);
            }
            return null;
        }
    }

    public static String getProxyFromRedis(UUID uuid) {
        try (Jedis jedis = ElementOriginLib.inst().getRedisIO().getJedisPool().getResource()) {
            return jedis.hget("redis-bungee::" + ElementOriginLib.inst().getLibConfig().getRedisBungeeNetworkId() + "::player::" + uuid + "::data", "proxy");
        }
    }

    public static String getServerFromRedis(UUID uuid) {
        try (Jedis jedis = ElementOriginLib.inst().getRedisIO().getJedisPool().getResource()) {
            return jedis.hget("redis-bungee::" + ElementOriginLib.inst().getLibConfig().getRedisBungeeNetworkId() + "::player::" + uuid + "::data", "server");
        }
    }

    public static String getLastServerFromRedis(UUID uuid) {
        try (Jedis jedis = ElementOriginLib.inst().getRedisIO().getJedisPool().getResource()) {
            return jedis.hget("redis-bungee::" + ElementOriginLib.inst().getLibConfig().getRedisBungeeNetworkId() + "::player::" + uuid + "::data", "last-server");
        }
    }

    public static InetAddress getIpAddressFromRedis(UUID uuid) {
        try (Jedis jedis = ElementOriginLib.inst().getRedisIO().getJedisPool().getResource()) {
            String ip = jedis.hget("redis-bungee::" + ElementOriginLib.inst().getLibConfig().getRedisBungeeNetworkId() + "::player::" + uuid + "::data", "ip");
            if (ip == null) return null;
            return InetAddresses.forString(ip);
        }
    }

    public static long getLastOnlineFromRedis(UUID uuid) {
        try (Jedis jedis = ElementOriginLib.inst().getRedisIO().getJedisPool().getResource()) {
            String unixString = jedis.hget("redis-bungee::" + ElementOriginLib.inst().getLibConfig().getRedisBungeeNetworkId() + "::player::" + uuid + "::data", "last-online");
            if (unixString == null) return -1;
            return Long.parseLong(unixString);
        }
    }

    public static boolean isPlayerTrulyOnProxy(String proxyId, UUID uuid) {
        try (Jedis jedis = ElementOriginLib.inst().getRedisIO().getJedisPool().getResource()) {
            return jedis.sismember("redis-bungee::" + ElementOriginLib.inst().getLibConfig().getRedisBungeeNetworkId() + "::proxies::" + proxyId + "::online-players", uuid.toString());
        }
    }

    public static Set<UUID> getProxyMembers(String proxyId) {
        try (Jedis jedis = ElementOriginLib.inst().getRedisIO().getJedisPool().getResource()) {
            Set<String> uuidsStrings = jedis.smembers("redisbungee::" + ElementOriginLib.inst().getLibConfig().getRedisBungeeNetworkId() + "::proxies::" + proxyId + "::online-players");
            HashSet<UUID> uuids = new HashSet<>();
            for (String proxyMember : uuidsStrings) {
                uuids.add(UUID.fromString(proxyMember));
            }
            return uuids;
        }
    }

}
